#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var chalk = _interopDefault(require('chalk'));
var program = _interopDefault(require('commander'));
var fileSize = _interopDefault(require('filesize'));
var path = require('path');
var prettyMs = _interopDefault(require('pretty-ms'));
var readPkg = _interopDefault(require('read-pkg'));
var __ = require('..');
var fs = require('fs');
var rollup = require('rollup');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var version = "0.12.2";

function createNext(arr, callback) {
  var index = 0;

  var next = function next() {
    if (!(index in arr)) {
      return;
    }

    var currentIndex = index++;
    var item = arr[currentIndex];
    callback(item, next, currentIndex);
  };

  return next;
}

function oneByOne(arr, callback) {
  var next = createNext(arr, callback);
  next();
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _continue(value, then) {
  return value && value.then ? value.then(then) : then(value);
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function build(configs, callbacks) {
  if (callbacks.start) {
    callbacks.start();
  }

  var cache = {};
  oneByOne(configs, _async(function (config, next, index) {
    var input = config.input,
        outputOptions = config.output;
    var output = outputOptions.file,
        format = outputOptions.format;
    var isModuleBuild = format === "cjs" || format === "es";
    var cacheKey = (isModuleBuild ? "module:" : "browser:") + input;
    config.cache = cache[cacheKey];

    if (callbacks.buildStart) {
      callbacks.buildStart(input, output);
    }

    return _continue(_catch(function () {
      var currentTime = Date.now();
      return _await(rollup.rollup(config), function (buildResult) {
        config.cache = cache[cacheKey] = buildResult.cache;
        return _await(buildResult.write(outputOptions), function () {
          var _statSync = fs.statSync(output),
              size = _statSync.size;

          var totalTime = Date.now() - currentTime;

          if (callbacks.buildEnd) {
            callbacks.buildEnd(output, size, totalTime);
          }
        });
      });
    }, function (err) {
      if (callbacks.error) {
        callbacks.error(err);
      }
    }), function () {
      if (index + 1 >= configs.length) {
        if (callbacks.end) {
          callbacks.end();
        }
      }

      next();
    });
  }));
}

function watch(configs, callbacks) {
  var watcher = rollup.watch(configs);
  watcher.on("event", function (event) {
    var code = event.code,
        input = event.input,
        output = event.output,
        duration = event.duration,
        error = event.error;

    if (callbacks.start && code === "START") {
      callbacks.start();
    }

    if (callbacks.end && code === "END") {
      callbacks.end();
    }

    if (callbacks.buildStart && code === "BUNDLE_START") {
      for (var _iterator = output, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var filename = _ref;
        callbacks.buildStart(input, filename);
      }
    }

    if (callbacks.buildEnd && code === "BUNDLE_END") {
      for (var _iterator2 = output, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var _filename = _ref2;

        var _statSync = fs.statSync(_filename),
            size = _statSync.size;

        callbacks.buildEnd(_filename, size, duration || 0);
      }
    }

    if (callbacks.error && (code === "ERROR" || code === "FATAL")) {
      callbacks.error(error);
    }
  });
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var bundlib = _async$1(function (cwd, options, callbacks, pkg) {
  var _temp = options.watch ? watch : build;

  return _await$1(__.configsFromPkg(cwd, options, pkg), function (_configsFromPkg) {
    _temp(_configsFromPkg, callbacks);
  });
});

function _async$1(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function log(msg) {
  console.log(msg);
}
function logError(err) {
  if (err.stack) {
    console.error(err.stack);
  }

  console.error(chalk.bgRed.bold(" error ") + " " + (err.message || err));
}

function _await$2(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var action = _async$2(function () {
  if (!chalk.level && !chalk.enabled && process.env.MINGW_CHOST && process.env.WINDIR) {
    chalk.level = 3;
    chalk.enabled = true;
  }

  var cwd = process.cwd();
  var dev = program.dev,
      watch = program.watch,
      silent = program.silent;
  var pkg;
  return _invoke(function () {
    if (!silent) {
      var msgVersion = chalk.green.bold("Bundlib v" + version);
      var msgReading = chalk.cyan.bold("reading package.json...");
      log(msgVersion + "\n");
      log(msgReading + "\n");
      return _await$2(readPkg({
        cwd: cwd,
        normalize: false
      }), function (_readPkg) {
        pkg = _readPkg;
        var _pkg = pkg,
            name = _pkg.name,
            ver = _pkg.version;

        if (name && ver) {
          var msgBuilding = chalk.green.bold("building:");
          var projInfo = chalk.yellow.bold(name + " v" + ver);
          log(msgBuilding + " " + projInfo + "\n");
        }
      });
    }
  }, function () {
    var buildIndex = 0;
    var callbacks = {
      error: function error(err) {
        logError(err);
      }
    };

    if (!silent) {
      _extends(callbacks, {
        buildEnd: function buildEnd(filename, size, duration) {
          var colorTag = chalk.green.bold.inverse(" built ");
          var colorFilename = chalk.yellow.bold(path.relative(cwd, filename));
          var colorSize = chalk.magenta.bold(fileSize(size));
          var colorIn = chalk.cyan("in");
          var colorTime = chalk.magenta.bold(prettyMs(duration, {
            secondsDecimalDigits: 2
          }));
          log(colorTag + " " + colorFilename + " ( " + colorSize + " " + colorIn + " " + colorTime + " )");
        },
        error: function error(err) {
          logError(err);
        }
      });

      if (watch) {
        _extends(callbacks, {
          start: function start() {
            if (buildIndex) {
              var msgRebuild = chalk.cyan("rebuilding...");
              log(msgRebuild + "\n");
            }

            buildIndex++;
          },
          end: function end() {
            var msgWait = chalk.cyan("waiting for changes...");
            log("\n" + msgWait);
          }
        });
      }
    }

    return _continueIgnored(_catch$1(function () {
      return _awaitIgnored(bundlib(cwd, {
        dev: dev,
        watch: watch
      }, callbacks, pkg));
    }, function (err) {
      logError(err);
    }));
  });
});

function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _catch$1(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _continueIgnored(value) {
  if (value && value.then) {
    return value.then(_empty);
  }
}

function _invoke(body, then) {
  var result = body();

  if (result && result.then) {
    return result.then(then);
  }

  return then(result);
}

function _async$2(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

program.version(version, "-v, --version").option("-d, --dev", "create development builds").option("-w, --watch", "run bundlib in watch mode").option("-s, --silent", "prevent messages from showing in the console").action(action).parse(process.argv);
//# sourceMappingURL=bundlib-cli.js.map
